@@ -21,11 +21,9 @@
 
 from __future__ import absolute_import
 
-import os.path
 from argparse import ArgumentParser
 
 from gevent import monkey; monkey.patch_all()
-from gevent import sleep
 
 from . import __version__
 from .state import SlimtaState
@@ -36,6 +34,8 @@ def parse_args():
     argparser.add_argument('--version', action='version', version='%(prog)s '+__version__)
     argparser.add_argument('-c', '--config', metavar='FILE', default=None,
                     help='Specifies a configuration file to read. If not given, the default locations ($HOME/.slimta/slimta.conf, /etc/slimta/slimta.conf) are checked.')
+    argparser.add_argument('-n', '--process-name', metavar='NAME', default=None,
+                    help='Use the process sub-section NAME for configuration. By default, the name of the executable is used.')
     argparser.add_argument('-a', '--no-daemon', dest='attached', action='store_true',
                     help='Override configs and force the process to remain attached to the terminal.')
     argparser.add_argument('-d', '--daemon', dest='attached', action='store_false',
@@ -47,33 +47,17 @@ def parse_args():
     return argparser, argparser.parse_args()
 
 
-def slimta():
-    state = SlimtaState('slimta')
+def main():
+    state = SlimtaState()
     state.load_config(*parse_args())
 
-    state.start_everything()
-
-    state.setup_logging()
-    state.redirect_streams()
-    state.daemonize()
-    sleep(0.1)
-    state.drop_privileges()
-
     state.loop()
 
 
-def worker():
-    state = SlimtaState('worker')
+def worker_main():
+    state = SlimtaState()
     state.load_config(*parse_args())
 
-    state.start_celery_queues()
-
-    state.setup_logging()
-    state.redirect_streams()
-    state.daemonize()
-    sleep(0.1)
-    state.drop_privileges()
-
     state.worker_loop()
 
 
