@@ -30,6 +30,7 @@ from functools import wraps
 from contextlib import contextmanager
 
 from config import Config, ConfigError, ConfigInputStream
+from gevent import sleep
 import slimta.system
 
 from .validation import ConfigValidation, ConfigValidationError
@@ -41,8 +42,8 @@ class SlimtaState(object):
     _global_config_files = [os.path.expanduser('~/.slimta/slimta.conf'),
                             '/etc/slimta/slimta.conf']
 
-    def __init__(self, program):
-        self.program = program
+    def __init__(self):
+        self.program = os.path.basename(sys.argv[0])
         self.args = None
         self.cfg = None
         self.edges = {}
@@ -59,22 +60,6 @@ class SlimtaState(object):
         finally:
             os.chdir(old_dir)
 
-    @contextmanager
-    def _with_pid_file(self):
-        if not self.args.pid_file:
-            yield
-        else:
-            pid_file = os.path.abspath(self.args.pid_file)
-            with open(pid_file, 'w') as f:
-                f.write('{0}\n'.format(os.getpid()))
-            try:
-                yield
-            finally:
-                try:
-                    os.unlink(pid_file)
-                except OSError:
-                    pass
-
     @contextmanager
     def _with_sighandlers(self):
         from signal import SIGTERM
@@ -103,6 +88,9 @@ class SlimtaState(object):
             return
         self.args = args
 
+        if args.process_name:
+            self.program = args.process_name
+
         files = self._global_config_files
         if args.config:
             files = [args.config]
@@ -142,6 +130,11 @@ class SlimtaState(object):
         elif not self.args.attached:
             slimta.system.daemonize()
 
+    def create_pid_file(self):
+        args_pid_file = self.args.pid_file
+        cfg_pid_file = self.cfg.process.get(self.program).get('pid_file')
+        return slimta.system.PidFile(args_pid_file or cfg_pid_file)
+
     def setup_logging(self):
         settings = self.cfg.process.get(self.program).get('logging')
         setup_logging(settings)
@@ -383,21 +376,39 @@ class SlimtaState(object):
 
     def worker_loop(self):
         from .celery import get_celery_worker
-        try:
-            with self._with_sighandlers():
-                with self._with_pid_file():
+
+        self.start_celery_queues()
+
+        self.setup_logging()
+        self.redirect_streams()
+        self.daemonize()
+        with slimta.system.PidFile(self.args.pid_file):
+            sleep(0.1)
+            self.drop_privileges()
+
+            try:
+                with self._with_sighandlers():
                     get_celery_worker(self.celery).run()
-        except (KeyboardInterrupt, SystemExit):
-            pass
+            except (KeyboardInterrupt, SystemExit):
+                pass
 
     def loop(self):
         from gevent.event import Event
-        try:
-            with self._with_sighandlers():
-                with self._with_pid_file():
+
+        self.start_everything()
+
+        self.setup_logging()
+        self.redirect_streams()
+        self.daemonize()
+        with self.create_pid_file():
+            sleep(0.1)
+            self.drop_privileges()
+
+            try:
+                with self._with_sighandlers():
                     Event().wait()
-        except (KeyboardInterrupt, SystemExit):
-            pass
+            except (KeyboardInterrupt, SystemExit):
+                pass
 
 
 # vim:et:fdm=marker:sts=4:sw=4:ts=4
